		package main;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import uiutils.ButtonPanel;
import uiutils.ComboSelectionPanel;
import uiutils.ConditionedPanel;
import uiutils.FileChooserPanel;
import uiutils.InputFieldPanel;
import uiutils.LabelPanel;
import uiutils.SimpleFrame;
import uiutils.SpinnerINTPanel;
import utils.CSV_Reader;
import utils.SFB_JSON_Util;
import utils.SFB_Util;

public class Main {
	private static final String XLSX_SESSION_PREFIX = "Session Date";
	private static final String SFB_RECORD_DATE = "Record Date";
	
	private static final String[] DEFAULT_HEADER_NAMES = new String[] {"Subject ID", "Subject Age", "Subject Sex", "Subject Type", "Extra Information"};
	
	public static void main(String[] args) {
		SimpleFrame frame = new SimpleFrame("Create Study", 600, 400);
		
		FileChooserPanel studyDestination = FileChooserPanel.CreateFolderChooser("Study Destination", ".", "Choose study destination folder", false);
		InputFieldPanel studyName = new InputFieldPanel("Name of the study: ", "Name");
		SpinnerINTPanel numberOfConsecutiveSubjects = new SpinnerINTPanel("Total of consecutively numbered subjects: ", 1, 1, 1000, 1);
		InputFieldPanel sfbCodes = new InputFieldPanel("SFB subjects: ", "sub-XXX, sub-XXX, sub-XXX, ...").setTextFieldprefWidth(300);
		ConditionedPanel useCustomSubjectCodes = new ConditionedPanel("Enter own subject codes:", false, sfbCodes);
		SpinnerINTPanel sessionAmount = new SpinnerINTPanel("Session amount: ", 1, 1, 100, 1);
		
		ComboSelectionPanel modalities = new ComboSelectionPanel("Modalities:", "Select", ComboSelectionPanel.SFB_MODALITIES);
		InputFieldPanel customeModalities = new InputFieldPanel("Modality Folder", "EDA, fMRI, ...").setTextFieldprefWidth(300);
		ConditionedPanel useCustomModalityFolders = new ConditionedPanel("Enter own modality folder names:", false, customeModalities);
		
		
		FileChooserPanel baseMetaJson = FileChooserPanel.CreateFileChooser("Base Meta.json", ".", "Choose base meta.json file", "meta.json", 0, false);
		FileChooserPanel xlsxImportFile = FileChooserPanel.CreateFileChooser("Excel import file", ".", "Choose base meta.json file", ".xlsx", 0, false);
		ConditionedPanel importFromXlsx = new ConditionedPanel("Import from Excel", false, baseMetaJson, xlsxImportFile);
		
		importFromXlsx.addActionListener( (ActionEvent e) -> {
			boolean notState = !importFromXlsx.getOutput();
			numberOfConsecutiveSubjects.setEnabled(notState);
				if (!notState)
			useCustomSubjectCodes.setValue(notState);
			sessionAmount.setEnabled(notState);
			
		});
		
		useCustomSubjectCodes.addActionListener( (ActionEvent e) -> {
			numberOfConsecutiveSubjects.setEnabled(true);
			if (useCustomSubjectCodes.getOutput())
				importFromXlsx.setValue(false);
			sessionAmount.setEnabled(true);
			
		});
		
		ButtonPanel submit = new ButtonPanel("Create", new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				File dest = studyDestination.getOutput().getFile();
				if (dest == null) {
					SimpleFrame.promt("Please select a destination folder.");
					return;
				}
				
				String fileName = studyName.getOutput();
				if (fileName == null || fileName.trim().isEmpty()) {
					SimpleFrame.promt("Please input a Study Name.");
					return;
				}
				fileName = fileName.trim();
				dest = new File(dest.getAbsolutePath() + "\\" + fileName);
				if (dest.exists()) {
					SimpleFrame.promt("Foldername already exists.");
					return;
				}
				String[] codes;
				CSV_Reader xlsxReader = null;
				
				int sessions = sessionAmount.getOutput();
				
				if (importFromXlsx.getOutput()) {
					if (baseMetaJson.getOutput() == null || baseMetaJson.getOutput().getFile() != null|| !baseMetaJson.getOutput().getFile().exists()) {
						SimpleFrame.promt("Please specify a valid base meta.json file");
						return;
					}
					
					if (xlsxImportFile.getOutput() == null || xlsxImportFile.getOutput().getFile() != null || !xlsxImportFile.getOutput().getFile().exists()) {
						SimpleFrame.promt("Please specify a valid excel import file");
						return;
					}
					
					xlsxReader = CSV_Reader.fromXLSX(xlsxImportFile.getOutput().getFile(), null);
					
					for(int i = 0; i < DEFAULT_HEADER_NAMES.length; i++) {
						if (!xlsxReader.getColumnHead(i).equals(DEFAULT_HEADER_NAMES[i])) {
							SimpleFrame.promt("Excel error: Column header Nr. " + i + " should be " + DEFAULT_HEADER_NAMES[i]);
							return;
						}
					}
					
					sessions = 0;
					while(xlsxReader.getMaxColumns() > sessions + DEFAULT_HEADER_NAMES.length && xlsxReader.getColumnHead(sessions + DEFAULT_HEADER_NAMES.length).equals(XLSX_SESSION_PREFIX + " " +  String.format("%02d", sessions + 1)))
						sessions++;

					if (sessions == 0) {
						SimpleFrame.promt("Please specify session dates in excel file: " + XLSX_SESSION_PREFIX + " 01");
						return;
					}
					
					xlsxReader.nullEntriesToEmptyStr();
					
					codes = xlsxReader.getCompleteColumn(0); // 0 for Subject ID
					
				} else {
					
					//Does not generate codes when a xlsx file is given!
					if (useCustomSubjectCodes.getOutput()) {
						String codesSTR = sfbCodes.getOutput();
						if (codesSTR.trim().isEmpty()) {
							SimpleFrame.promt("Please enter patient identifiers");
							return;
						}
						codesSTR = codesSTR.replaceAll(" ", "");
						codes = codesSTR.split(",");
					} else {
						int subj = numberOfConsecutiveSubjects.getOutput();
						codes = new String[subj];
						for(int i = 0; i < subj; i++) {
							codes[i] = "sub-" + (i + 1);
						}
					}
				}
				
				
				
				
				String[] modalityFolders = modalities.getOutput();
				
				if (useCustomModalityFolders.getOutput()) {
					String modalitySTR = customeModalities.getOutput();
					if (modalitySTR.trim().isEmpty()) {
						SimpleFrame.promt("Please enter custom modalities");
						return;
					}
					modalitySTR = modalitySTR.replaceAll(" ", "");
					String[] newModalityFolders = modalitySTR.split(",");
					
					String[] combined = new String[newModalityFolders.length + modalityFolders.length];
					for(int i = 0; i < modalityFolders.length; i++)
						combined[i] = modalityFolders[i];
					for(int i = 0; i < newModalityFolders.length; i++)
						combined[i + modalityFolders.length] = newModalityFolders[i];
					
					modalityFolders = combined;
					
				}
				
				try {
					File baseMetaJsonFile = null;
					if (baseMetaJson.getOutput() != null)
						baseMetaJsonFile = baseMetaJson.getOutput().getFile();
					createStudy(dest, codes, sessions, modalityFolders, xlsxReader, baseMetaJsonFile);
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				
				SimpleFrame.promt("Finished");
				
			}
		});
		
		frame.addCenter(new LabelPanel("Study Creator", 30));
		frame.addCenter(studyDestination);
		frame.addCenter(studyName);
		frame.addCenter(numberOfConsecutiveSubjects);
		frame.addCenter(useCustomSubjectCodes);
		frame.addCenter(sessionAmount);
		frame.addCenter(modalities);
		frame.addCenter(useCustomModalityFolders);
		frame.addCenter(importFromXlsx);
		frame.addCenterNoLeftalignment(submit);
		
		frame.submit();
	}
	
	private static void createStudy(File studyFile, String[] sfbCodes, int sessionAmount, String[] modalities, CSV_Reader xlsxReader, File baseMeta) throws IOException {
		studyFile.mkdir();
		for(String code : sfbCodes) {
			File subjFolder = new File(studyFile.getAbsolutePath() + "\\" + code);
			subjFolder.mkdir();
			
			for(int i = 1; i <= sessionAmount; i++)  {
				File sessionFolder = new File(subjFolder.getAbsolutePath() + "\\ses-" + String.format("%02d", i));
				sessionFolder.mkdir();
				
				if (xlsxReader != null)
					createMetaFile(sessionFolder, xlsxReader, baseMeta, code, i);
				
				for(String modality : modalities) {
					File modFolder = new File(sessionFolder.getAbsolutePath() + "\\" + modality);
					modFolder.mkdir();
					if (xlsxReader != null)
						createMetaFile(modFolder, xlsxReader, baseMeta, code, i);
				}
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	private static void createMetaFile(File dir, CSV_Reader xlsxReader, File baseMeta, String sfbSubjCode ,int currentSession) throws IOException {
		File metaJson = new File(dir.getAbsolutePath() + File.separator + "meta.json");
		
		SFB_Util.copy(baseMeta, metaJson);
		
		SFB_JSON_Util.performOnJson(metaJson, (obj) -> {
			String[] currentRow = xlsxReader.getCompleteRow(xlsxReader.findRow(sfbSubjCode, 0)); // SubjID
			
			 //new String[] {"Subject ID", "Subject Age", "Subject Sex", "Subject Type", "Extra Information"}
			 obj.put("Subject ID", currentRow[0]);
			 obj.put("Subject age", currentRow[1]);
			 obj.put("Subject sex", SFB_JSON_Util.getGenderFromStringStart(currentRow[2]).toString());
			
			 if (currentRow[3].equals("patient")) {
				 currentRow[3] = "Patient";
			 }
			 if (currentRow[3].toLowerCase().equals("healthy control subject")) {
				 currentRow[3] = "Healthy control subject";
			 }
			 
			 if (!currentRow[3].equals("Healthy test subject") && !currentRow[3].equals("Patient") && !currentRow[3].equals("Healthy control subject"))
				 currentRow[3] = "Healthy test subject";
			 
			 obj.put("Subject type", currentRow[3]);
			 obj.put("Extra information", currentRow[4]);
			
			for(int i = DEFAULT_HEADER_NAMES.length; i < currentRow.length; i++) {
				String header = xlsxReader.getColumnHead(i);
				if (header.startsWith(XLSX_SESSION_PREFIX)) {
					if (header.endsWith("" + String.format("%02d", currentSession))) { //Identifies the current session. This breaks in case of more than 9 sessions. But this wont happen.
						obj.put("Record date", currentRow[i]);
						obj.put("DataCite-Date", currentRow[i]);
						obj.put("DublinCore-Date", currentRow[i]);
					}else
						continue;
				}
			
			}
		});
		
	}
}
